% this reads ui and sets global parameters for ALI

 % key parameters already obtained: N_f, c, N_c

 % get trace color
 ui_color;

 % remember old parameters 
 if exist('f_des')&exist('mse_opt')&exist('ranges'),
   f_des_old = f_des;
   mse_opt_old = mse_opt;
   ranges_old = ranges;
 else
   f_des_old = NaN*ones(N_f,1);
   mse_opt_old = NaN;
   ranges_old = [NaN,NaN];
 end

 % get remaining channel params
 SNR = str2num(get(h_SNR,'String'));
 if (~isreal(SNR))|(max(size(SNR)) ~= 1),
   error('SNR must be a real-valued scalar or "Inf"');
 elseif SNR == Inf, 
   SNR = 10000; 
 end

 % max parameter length
 N_p = max(N_f,N_c);

 % get source params (assume eval_max = 1)
 %if get(h_source,'Value') == 1,	eval_min = 1;		% white
 %elseif get(h_source,'Value') == 2, eval_min = 0.5;	% colored #1
 %elseif get(h_source,'Value') == 3, eval_min = 0.1;	% colored #2
 %end
 %evals = [1,eval_min,(N_p-1-eval_min)/(N_p-2)*ones(1,N_p-2)];	

 % get any set of unitary evecs to build the source autocovariance mtx 
 %[Vp,dum] = eig(toeplitz(evals));	% evals is the right size, thats all...
 %Rss_p = Vp*diag(evals)*Vp'; 		% now has correct eval spread 
 %Rss_p = Rss_p/Rss_p(1,1);		% normalize it so that sigma2_s = 1

 % get source params
 if get(h_source,'Value') == 1,		% white
   A_s = [1]; B_s = [1];		% ARMA coefs
   Rss_p = eye(N_p);
 elseif get(h_source,'Value') == 2,	% AR 1
   A_s = [1 0.3];			
   B_s = [sqrt(1-A_s(2)^2)];		
   Rss_p = toeplitz((B_s(1)^2)/(1-A_s(2)^2)*(-A_s(2)).^[0:N_p-1]);
 elseif get(h_source,'Value') == 3,	% MA 2
   A_s = [1];
   B_s = [1 2 1]; B_s = B_s/norm(B_s);
   Rss_p = convmtx(B_s(:),N_p)'*convmtx(B_s(:),N_p);
 elseif get(h_source,'Value') == 4,	% sinusoidal
   f_s = 0.1;	% normalized to Fs
   Rss_p = toeplitz(cos(2*pi*f_s*[0:N_p-1]));
 elseif get(h_source,'Value') == 5,	% DC
   Rss_p = ones(N_p);
 end

 % determine optimal delay (undermodelled case), mse_opt, and f_des
 sigma2_n = 10^(-SNR/10);
 Rss_f = Rss_p(1:N_f,1:N_f);		% equalizer-sized source autocov.
 if N_f < N_c,
   C = c*ones(1,N_c-N_f+1);		% a duplication of channels
   for i=1:N_c-N_f+1,
     C([i:i+N_f-1],i) = zeros(N_f,1);	% temporarily zero-fill
     f_tilde = -pinv(Rss_f)*Rss_p([i:i+N_f-1],:)*C(:,i);  % use temp zf
     C([i:i+N_f-1],i) = f_tilde;	% optimal difference from channel
   end 
   E = C'*Rss_p*C + sigma2_n*eye(N_c-N_f+1);	% the resulting MSE matrix
   [mse_opt,delta_opt] = min(diag(E));
   f_des = c([delta_opt:delta_opt+N_f-1])...
	   - C([delta_opt:delta_opt+N_f-1],delta_opt);
 else
   mse_opt = sigma2_n;			% perfect estimation, save noise
   delta_opt = 1;
   f_des = [c;zeros(N_f-N_c,1)];
 end

 % avoid warnings about log of zero
 if mse_opt < eps^2, mse_opt = eps^2; end;

 % eigenanalysis
 [V,Lam] = eig(Rss_f);			% get the two-tap evecs
 vec1 = V(:,1)*sqrt(1/max(Lam(1,1),eps));	% ellipse axis
 vec2 = V(:,2)*sqrt(1/max(Lam(2,2),eps));	% ellipse axis
 ranges = max(abs([vec1,vec2].')).';	% parameter range for mse surface

 % see if the channel or source has changed significantly
 has_changed = 0;
 if (N_f ~= length(f_des_old)),
   has_changed = 2;			% change in surface and mse
 elseif (f_des ~= f_des_old)|(ranges ~= ranges_old)
  has_changed = 2;			% change in surface and mse
 elseif (mse_opt ~= mse_opt_old),
   has_changed = 1;			% change only in mse
 end

 % clear {eigenstuff,surface,mse-bound,trajectories} if needed 
 if has_changed == 2,			% change in surface and mse
   set(h_sprd,'String','');
   if ishandle(h_trace_g), delete(h_trace_g); end; h_trace_g = [];
   if ishandle(h_trace_c), delete(h_trace_c); end; h_trace_c = [];
   if ishandle(h_trace_t), delete(h_trace_t); end; h_trace_t = [];
   if ishandle(h_trace_mse), delete(h_trace_mse); end; h_trace_mse = [];
   for i=1:length(h_trace_ff),
     if ishandle(h_trace_ff(i)), delete(h_trace_ff(i)); end;
   end
   h_trace_ff = [];
 elseif has_changed == 1,		% change only in mse
   if ishandle(h_trace_mse), delete(h_trace_mse); end; h_trace_mse = [];
 end
