% this handles the "calc trajectory" action for ALI

 if ishandle(h_trace_c), 	% initialize via ginput on surface
     set(h_msg,'BackgroundColor',[0 1 0]);
     set(h_msg,'FontAngle','italic', 'String','click on surface...'); drawnow;
     set(fig_menu,'HandleVisibility','off');
     figure(fig_surf); zoom off; f_init = ginput(1).'; zoom on;	% mouseclick
     set(fig_menu,'HandleVisibility','on');
     set(h_msg,'BackgroundColor',[.8 .8 .8]);
     set(h_msg,'FontAngle','normal', 'String',''); drawnow;
 else,				% initialize at origin 
   f_init = zeros(N_f,1);
 end

 % get the relevant parameters from ui
 N = str2num(get(h_num,'String'));              % length of input sequence
   if N<2, error('must be at least two iterations '); end;
 mu = str2num(get(h_step,'String'));            % algorithm step size

 % setup, allowing for arbitrarily long equalizers
 if get(h_source,'Value') < 4,			% ARMA (A_s,B_s s.t. sigma2_s=1)
   s = filter(B_s,A_s,randn(1,N+N_p-1));
 elseif get(h_source,'Value') == 4,		% sinusoidal
   s = sqrt(2)*cos(2*pi*f_s*[0:N+N_p-2]);
 elseif get(h_source,'Value') == 5,		% DC
   s = ones(1,N+N_p-1);
 end
 r = conv(c,s);
 r = r(N_p:N_p+N-1)+sqrt(sigma2_n)*randn(1,N);	% measurement 
 f = zeros(N_f,N);				% equalizer history
 err = zeros(1,N);				% error history

 % initialize
 f(:,1) = f_init;	
 delay_reg = delta_opt - N_p;			% regressor delay
 set(h_msg,'FontAngle','italic', 'String','calculating trajectory...'); 
 drawnow;

 % implement one of various algorithms
 alg = get(h_alg,'Value');
 if alg == 1,		% straight-up LMS
   for i = 1:N,
     reg = s(1,i-delay_reg:-1:i-delay_reg-N_f+1); 
     err(i) = reg*f(:,i) - r(i);
     f(:,i+1) = f(:,i)-mu*reg.'*err(i);
   end
 elseif alg == 2,	% normalized LMS (N-LMS)
   alpha = 0.001;
   for i = 1:N,
     reg = s(1,i-delay_reg:-1:i-delay_reg-N_f+1); 
     err(i) = reg*f(:,i) - r(i);
     reg_norm = norm(reg);
     f(:,i+1) = f(:,i)-mu/(alpha+reg_norm)*reg.'*err(i);
   end
 elseif alg == 3,	% signed-error LMS (SE-LMS)
   for i = 1:N,
     reg = s(1,i-delay_reg:-1:i-delay_reg-N_f+1); 
     err(i) = reg*f(:,i) - r(i);
     f(:,i+1) = f(:,i)-mu*reg.'*sign(err(i));
   end
 elseif alg == 4,	% signed-regressor LMS (SE-LMS)
   for i = 1:N,
     reg = s(1,i-delay_reg:-1:i-delay_reg-N_f+1); 
     err(i) = reg*f(:,i) - r(i);
     f(:,i+1) = f(:,i)-mu*sign(reg).'*err(i);
   end
 elseif alg == 5,	% signed-error, signed-regressor LMS (SS-LMS)
   for i = 1:N,
     reg = s(1,i-delay_reg:-1:i-delay_reg-N_f+1); 
     err(i) = reg*f(:,i) - r(i);
     f(:,i+1) = f(:,i)-mu*sign(reg).'*sign(err(i));
   end
 elseif alg == 6,	% leaky LMS (L-LMS)
   leak = mu/100;
   for i = 1:N,
     reg = s(1,i-delay_reg:-1:i-delay_reg-N_f+1); 
     err(i) = reg*f(:,i) - r(i);
     f(:,i+1) = (1-leak)*f(:,i)-mu*reg.'*err(i);	
   end
 elseif alg == 7,	% stepsize reduction (gearshifted) LMS (G-LMS)
   thresh_dB = 5;		% dB above mse_opt where gearshift happens
   gearstep = 10;		% amount to reduce stepsize by
   thresh = 10^(thresh_dB/20)*sqrt(mse_opt);
   mu_new = mu;
   for i = 1:N,
     reg = s(1,i-delay_reg:-1:i-delay_reg-N_f+1); 
     err(i) = reg*f(:,i) - r(i);
     if (i>10)&(mu_new==mu), 	% onetime reduction in stepsize
       if max(abs(err(i-9:i)))<thresh, 	% need at least 10 good error values
	 mu_new = mu/gearstep; 
         disp(['reduced stepsize by ',num2str(gearstep),...
		' at iteration ',num2str(i),'.']);
       end;
     end
     f(:,i+1) = f(:,i)-mu_new*reg.'*err(i);
   end
 elseif alg == 8,	% pure gradient descent (MSE-GD)
   for i = 1:N,
     err(i) = sqrt( (f_des-f(:,i))'*Rss_f*(f_des-f(:,i)) + mse_opt );
     f(:,i+1) = f(:,i) + mu*Rss_f*(f_des-f(:,i));
   end
 end

 % error history, with smoothing 
 [filt_num,filt_den] = butter(1,10^(-get(h_smooth,'Value')));
 e_filt = 10*log10(abs(filtfilt(filt_num,filt_den,err(1:length(err)-1).^2)));
 D_e = 1;	% data decimation factor

 % plot trajectories on cost surface and set color
 if ishandle(h_trace_c), % get initialization from surface
   figure(fig_surf);
   hold on; h_trace_new = plot(f(1,:),f(2,:)); hold off;
   set(h_trace_new,'Color',color);
   h_trace_t = [h_trace_t,h_trace_new];
 end; 

 % plot error traces in a separate figure
 figure(fig_hist); 
 set(fig_hist, 'DeleteFcn','ui_del_hist');
 if ishandle(h_trace_h),
   axe = axis; 				% retain axis properties 
   hold on;				% dont clobber traces
 else
   axe = [0,0,Inf,-Inf];		% force new axis, allow new traces
 end
 h_trace_new = plot([1:length(e_filt)]*D_e,e_filt);
 hold off;
 set(h_trace_new,'Color',color); 
 h_trace_h = [h_trace_h,h_trace_new];

 % always replot MMSE boundary (since length may have changed)
 if exist('N_max'), N_max = max(N_max,N); else, N_max = N; end;
 if ishandle(h_trace_mse), delete(h_trace_mse); end;
 hold on;
 h_trace_mse = plot([0:N_max-1],...	% avoid log of zero
	ones(1,N_max)*10*log10(abs(mse_opt)),'r:');
 hold off;
 if mse_opt < 10*eps, 
   disp(['Warning: calculated MSE bound of ',...
	num2str(10*log10(abs(mse_opt)),3),...
	' dB is unreliable due to numerical precision']);
 end;

 % set axis to avoid clipping old traces
 axis([0,N_max,min([axe(3),1.1*e_filt,1.1*10*log10(abs(mse_opt))]),...
	max([axe(4),e_filt+5])]);
 title('Smoothed squared-error history'); 
 xlabel('iteration number'); ylabel('dB');
 h_trace_mse = [h_trace_mse, legend(h_trace_mse,'MSE bound',4)]; 
 zoom on;

 % clear user message and enable clear,freq
 set(h_msg,'FontAngle','normal', 'String','');
 set(h_clr,'Enable','on');

